from datetime import datetime
from typing import List, Optional
from pydantic import BaseModel
from uuid import UUID



class ItemModel(BaseModel):
    _id: UUID
    seq: int
    id: UUID
    name: str
    title: str
    caption: str
    icon: str
    path: str
    route: str
    tags: Optional[str]
    perms: Optional[List[str]]


class MenuModel(BaseModel):
    _id: UUID
    node: str
    seq: int
    id: UUID
    name: str
    title: str
    caption: str
    icon: str
    path: str
    route: str
    colour: str
    tags: Optional[str]
    items: Optional[List[ItemModel]]


class NavModel(BaseModel):
    node: str
    menus: List[MenuModel]

    
class AccountModel(BaseModel):
    id: Optional[UUID] = None
    name: Optional[str] = None
    navs: Optional[List[NavModel]] = None
    uix: dict
    tags: Optional[str] = None
    is_active: bool = False
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    


from collections import defaultdict
from typing import List, Optional
from uuid import UUID

from django.conf import settings
from django.http import HttpRequest

from nkunyim_util.caches.account_cache import AccountCache
from nkunyim_util.models.account_model import AccountModel, MenuModel, NavModel
from nkunyim_util.api.nkunyim_api_client import NkunyimApiClient
from nkunyim_util.models.user_model import RoleModel
from nkunyim_util.services.session_service import SessionService


from .signals import access_data_updated


class AccountService:
    
    def __init__(self, req: HttpRequest, session_key: str, application_id: UUID) -> None:
        self.cache = AccountCache(key=f"acc.{session_key}")
        self.application_id = str(application_id)
        self.req = req


    def _build_from_menus(self, menus: List[dict]) -> tuple[dict[str, str], List[NavModel]] :
        env = "dev"
        uix = {}
        menus_dict = defaultdict(List[MenuModel])

        for menu in menus:
            node = menu['node']
            module = menu['module']
            menu_data = {
                '_id': menu['id'],
                'node': node,
                'seq': menu['seq'],
                **{key: module[key] for key in (
                    'id', 'name', 'title', 'caption', 'icon', 'path', 'route', 'colour', 'tags'
                )}
            }
            items = menu['items']

            module_name = str(module['name']).title()
            module_path = str(module['path']).lower()
            
            uix[f"{module_name}Page"] = f"./{module_path}/home.{env}"

            menu_data['items'] = []
            for item in items:
                page = item['page']
                uix[f"{module_name}{str(page['name']).title()}Page"] = f"./{module_path}/{str(page['path']).lower()}.{env}"
                item_data = {
                    '_id': menu['id'],
                    'seq': menu['seq'],
                    **{key: page[key] for key in (
                        'id', 'name', 'title', 'caption', 'icon', 'path', 'route', 'tags'
                    )}
                }
                menu_data['items'].append(item_data)
                
            if node in {"dashboard", "modules", "account", "system"}:
                menus_dict[node].append(MenuModel(**menu_data))

        navs = [
            NavModel(node=str(nodex).title(), menus=menux)
            for nodex, menux in menus_dict.items()
        ]

        return uix, navs
        
        
    def _get_account_from_api(self) -> Optional[dict]:
        try:
            client = NkunyimApiClient(req=self.req, name=settings.MARKET_SERVICE)
            response = client.get(path=f"/accounts/menus/?application_id={self.application_id}")
            return dict(response.json()) if response.ok else None
        except:
            return None
        
        
    def _make(self) -> AccountModel:
        navs = None
        uix = dict(settings.NKUNYIM_UIX)
        account = self._get_account_from_api()
        if account and 'id' in account and account['id']:
            navs_uix = self._build_from_menus(menus=account['menus'])
            nui, navs = navs_uix
            uix.update(nui)

        else:
            account = {}
        
        role = account.get('role', None)
        if role and 'id' in role and role['id']:
            session_service = SessionService(req=self.req)
            user = session_service.get_user()
            if user :
                user.role = RoleModel(**role)
                session_service.set_user(user_model=user)
            
        model_data = {
            **account,
            'navs': navs,
            'uix': uix
        }
        access_model = AccountModel(**model_data)
        self.cache.set(model=access_model, timeout=60 * 60 * 24)
        
        # Inform interested parties
        access_data_updated.send(sender=AccountModel, instance=access_model)
        
        return access_model
            
    
    def get(self, refresh: bool = False) -> AccountModel:
        return self._make() if refresh else self.cache.get() or self._make()




from typing import List, Optional
from pydantic import BaseModel
from uuid import UUID


class ModuleModel(BaseModel):
    id: UUID
    name: str
    title: str
    version: str


class ProjectModel(BaseModel):
    id: UUID
    name: str
    caption: str
    description: str
    logo_url: str
    is_locked: bool
    tags: str


class ApplicationModel(BaseModel):
    id: UUID
    mode: str
    name: str
    title: str
    caption: str
    description: str
    keywords: str
    image_url: str
    logo_url: str
    logo_light_url: str
    logo_dark_url: str
    icon_url: str
    icon_light_url: str
    icon_dark_url: str
    domain: str
    tags: str
    colour: str
    is_active: bool
    project: ProjectModel
    modules: Optional[List[ModuleModel]]



import os
from typing import Union

from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import hashes, serialization

from django.conf import settings


class RSAEncryption:


    def load_private_pem(self) -> bytes:
        private_pem = None
        with open(os.path.join(settings.BASE_DIR, 'nkunyim_rsa/private_key.pem'), mode='rb') as private_file:
            private_pem = private_file.read()

        return private_pem
    
    
    def load_public_pem(self, name:str) -> bytes:
        file_name = name.lower()
        public_pem = None
        with open(os.path.join(settings.BASE_DIR, f"nkunyim_rsa/{file_name}_key.pem"), mode='rb') as public_file:
            public_pem = public_file.read()

        return public_pem
    
    
    def get_private_key(self) -> rsa.RSAPrivateKey:
        private_key = serialization.load_pem_private_key(
            self.load_private_pem(),
            password=str.encode(settings.RSA_PRIVATE_PHRASE)
        )
        return private_key # type: ignore

    
    def get_public_key(self, name: Union[str, None] = None) -> rsa.RSAPublicKey:
        if name:
            public_pem = self.load_public_pem(name=name)
            public_key = serialization.load_pem_public_key(public_pem)
        else:
            private_key = self.get_private_key()
            public_key = private_key.public_key()
            
        return public_key # type: ignore
    

    def encrypt(self, plain_text: str, name: Union[str, None] = None) -> bytes:
        
        public_key = self.get_public_key(name=name)
        
        cipher_text = public_key.encrypt(
            plain_text.encode(),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )

        return cipher_text


    def decrypt(self, cipher_text: bytes) -> str:
        private_key = self.get_private_key()
        plain_text = private_key.decrypt(
            cipher_text,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )
        return plain_text.decode()



from datetime import datetime
from pydantic import BaseModel
from typing import Optional

from nkunyim_util.models.user_model import UserModel


class TokenModel(BaseModel):
    access_token: str
    token_type: str
    expires_at: datetime


class SessionModel(BaseModel):
    token: Optional[TokenModel]
    user: Optional[UserModel]